#ifndef CHKTGPRP_H
#define CHKTGPRP_H
#include <atlbase.h>
#include <assert.h>
#include "cvbool.h"
#include "itag.h"
#include "bstrtostr.h"

namespace rcutils {


template <class OBJ, class PROP>
inline bool is_prop(const OBJ& obj, PROP prop)
{
	CComVariant varValue;
	HRESULT hCode = obj->GetProperty(prop, varValue);
	assert(SUCCEEDED(hCode));
	assert(varValue.vt == VT_BOOL);
	return bool_(varValue);
}


inline bool is_prop(ITag* pTag, ULONG ulProp)
{
	return is_prop(pTag,ulProp);
}

template <class OBJ, class PROP>
static inline const std::string str_prop(const OBJ& obj, PROP prop)
{
	CComVariant varValue;
	HRESULT hCode=obj->GetProperty(prop,varValue);
	assert(SUCCEEDED(hCode));
	assert(varValue.vt == VT_BSTR);
	if(FAILED(hCode) || varValue.vt != VT_BSTR) {
		return string("");
	}
	return BstrToString(varValue.bstrVal);
}


class CXComVariant : public CComVariant 
{
	public:
		operator ULONGLONG ()
		{
			return CComVariant::ullVal;
		}
		operator ULONG ()
		{
			return CComVariant::ulVal;
		}
		operator SHORT ()
		{
			return CComVariant::iVal;
		}

		operator long ()
		{
			return CComVariant::lVal;
		}

		operator double ()
		{
			return CComVariant::dblVal;
		}
		operator float ()
		{
			return CComVariant::fltVal;
		}
		operator bool ()
		{
			return bool_(CComVariant::boolVal);
		}



};

#ifdef _DEBUG

template<class T> class type_trail {};
template<> class type_trail<bool>      { public: static VARTYPE vt(){return VT_BOOL;};};
template<> class type_trail<double>    { public: static VARTYPE vt(){return VT_R8;  };};
template<> class type_trail<float>     { public: static VARTYPE vt(){return VT_R4;  };};
template<> class type_trail<ULONG>     { public: static VARTYPE vt(){return VT_UI4; };};
template<> class type_trail<long>      { public: static VARTYPE vt(){return VT_I4;  };};
template<> class type_trail<USHORT>    { public: static VARTYPE vt(){return VT_UI2; };};
template<> class type_trail<SHORT>     { public: static VARTYPE vt(){return VT_I2;  };};
template<> class type_trail<ULONGLONG> { public: static VARTYPE vt(){return VT_UI8; };};


#endif

template <class RET, class OBJ, class PROP>
inline const RET prop_(const OBJ& obj, PROP prop)
{
	CXComVariant varValue;
	HRESULT hCode = obj->GetProperty(prop,varValue);
#ifdef _DEBUG
	ASSERT(varValue.vt == type_trail<RET>::vt());
#endif
	assert(SUCCEEDED(hCode));
	return (RET)varValue;
}

template <class RET, class OBJ, class PROP, class ARG>
inline const RET propxa_(const OBJ& obj, PROP prop, const ARG& arg)
{
	CXComVariant varValue;
	HRESULT hCode = obj->GetPropertyEx(prop,varValue,arg);
#ifdef _DEBUG
	ASSERT(varValue.vt == type_trail<RET>::vt());
#endif
	assert(SUCCEEDED(hCode));
	return (RET)varValue;
}

template <class RET, class OBJ, class PROP>
inline const RET propx_(const OBJ& obj, PROP prop)
{
	return propxa_<RET,OBJ,PROP,long>(obj,prop,0L);
}



template<class T>
const T get_low_bit(T& v)
{
	T res=1;
	if(v==0)return 0;
	for(int i=0;v;++i){
		if((v&1)==1){
			v>>=1;
			v<<=(i+1);
			return res;
		}
		v>>=1;
		res<<=1;
	}
	return 0;
}

}

#endif