#ifndef IDATASOURCE_H
#define IDATASOURCE_H


// {D190EC18-7506-4249-A820-F73B8EA73222}
extern const GUID IID_IDataSource;



enum {
	DSTFLAG_INPUT      = 0x00000001, // m_pDevChan->isInputChannel()
	DSTFLAG_OUTPUT     = 0x00000002,  // m_pDevChan->isOutputChannel()
	DSTFLAG_IRREGULAR  = 0x00000004,
	DSTFLAG_UTS        = 0x00010000   // isUTSChannel
};

class IDataSource : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE Destroy() = 0;
	virtual HRESULT STDMETHODCALLTYPE GetTypeFlags(ULONG* a_pnFlags) const = 0;
	virtual HRESULT STDMETHODCALLTYPE GetPermanentId(BSTR* a_pbstrId) const = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFreq(double* a_pdblFreq) const = 0; // PROP_CHAN_FREQ
	
	/// PROP_CHAN_FREQ_COUNT
	virtual HRESULT STDMETHODCALLTYPE GetFreqCount(LONG* a_pnCount) const = 0;
	virtual HRESULT STDMETHODCALLTYPE GetFreqAt(double* a_pdblFreq, LONG a_nIndex) const = 0; // PROP_CHAN_FREQ
	virtual HRESULT STDMETHODCALLTYPE SetFreq(double a_dblFreq) = 0; // PROP_CHAN_FREQ

	virtual HRESULT STDMETHODCALLTYPE SetAlias(BSTR a_bstrAlias) = 0;
	
	///      
	/// GetProperty(PROP_START_OFFSET, &dblStartOffset);
	virtual HRESULT STDMETHODCALLTYPE GetStartOffset(double* a_pdblOffset) = 0;

	virtual HRESULT STDMETHODCALLTYPE GetAddress(BSTR* a_pbstrId) const = 0;

	
	/// GetProperty(PROP_INFO
	virtual HRESULT STDMETHODCALLTYPE GetInfo(BSTR* a_pbstrInfo) const = 0;
	
	/// @todo -   
	///       
	virtual HRESULT STDMETHODCALLTYPE LoadTransformerKoef() = 0;

	/// @todo -   
	///       
	virtual HRESULT STDMETHODCALLTYPE UpdateTransformerDB() = 0;

	/// GetFlashTareInfo
	virtual HRESULT STDMETHODCALLTYPE GetFlashTareInfo(BSTR* a_pInfo) const = 0;

	virtual HRESULT STDMETHODCALLTYPE GetSampleSize(LONG* a_pnSize) const = 0; // GetChanSize * 2

	////	EditObject(m_pDevChan->device, (HWND)a_dwParam)
	virtual HRESULT STDMETHODCALLTYPE EditOwner(HWND a_hWnd) = 0;


	virtual HRESULT STDMETHODCALLTYPE UploadTareToDevice() = 0;
	virtual HRESULT STDMETHODCALLTYPE WriteCalibratorName(BSTR a_bstrName) = 0;
	virtual HRESULT STDMETHODCALLTYPE DownloadTareFromDevice() = 0;

	//// GetProperty(PROP_CHAN_TRANSFORMER, &pTX)
	///       
	virtual HRESULT STDMETHODCALLTYPE GetTare(IUnknown** a_pObject) const = 0;

	/// PROP_CHAN_TRANSFORMER
	virtual HRESULT STDMETHODCALLTYPE SetTare(IUnknown* a_pObject) = 0;

	virtual HRESULT STDMETHODCALLTYPE ZBalance(double*) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetRange(double* dblMin, double* dblmax) const = 0; // PROP_RANGE_MIN PROP_RANGE_MAX

	/// GetType
	virtual HRESULT STDMETHODCALLTYPE GetInternalType(ULONG* a_pnType) const = 0;

	/// GetChanNo
	virtual HRESULT STDMETHODCALLTYPE GetChanIndex(LONG* a_pnNo) const = 0;

	
	/// @todo        ,   ""
	/// PROP_CHAN_SERNO
	virtual HRESULT STDMETHODCALLTYPE GetOwnerSerial(LONG* a_pnNo) const = 0;
	
	// PROP_OWNER_NAME
	virtual HRESULT STDMETHODCALLTYPE GetOwnerName(BSTR* a_pName) const = 0;

	///     IUnknown???....
	virtual HRESULT STDMETHODCALLTYPE GetOwner(IUnknown** a_ppOwner) = 0;

	///  Code/Volts/...
	/// PROP_CHAN_TYPE_SIGNAL
	virtual HRESULT STDMETHODCALLTYPE SetSignalType(LONG nType) = 0;

	/// GetDataType
	virtual HRESULT STDMETHODCALLTYPE GetDataType(VARTYPE* vt) const = 0;


	// PushOutputValue
	virtual HRESULT STDMETHODCALLTYPE PushOutputValue(double dblVal) = 0;

	// PushOutputValuesArray
	virtual HRESULT STDMETHODCALLTYPE PushOutputValuesArray(ULONG a_nCount, double* a_pData) = 0;

	///PROP_CHAN_HANDLER_DATA, 0
	/// ...   DevApi...
	virtual HRESULT STDMETHODCALLTYPE SetChanHandlerData(void* a_pData) = 0;

	/// PROP_CHAN_HANDLER
	virtual HRESULT STDMETHODCALLTYPE SetChanHandler(void* a_pHandler) = 0;


	/// PROP_CHAN_T_BLOCK
	virtual HRESULT STDMETHODCALLTYPE SetDataBlockSize(ULONG a_nSize) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetDataBlockSize(ULONG* a_npSize) = 0;

	///RANGE_COUNT_CHAN
	virtual HRESULT STDMETHODCALLTYPE GetRangesCount(LONG* a_npCount) const = 0;

	/// RANGE_CHAN_MIN
	/// RANGE_CHAN_MAX
	/// GetProperty(RANGE_CHAN_MIN, a_pRangeMin, a_nIndex
	virtual HRESULT STDMETHODCALLTYPE GetRangeAt(double* pMin, double* pMax, LONG a_nIndex) const = 0;
	
	/// GetProperty(RANGE_CHAN_UNITS, zsLine)
	virtual HRESULT STDMETHODCALLTYPE GetRangeUnits(BSTR* a_pUnits, LONG a_nIndex) const = 0;

	///INDEX_RANGE_CHAN

	virtual HRESULT STDMETHODCALLTYPE SetRangeIndex(LONG a_nIndex) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetRangeIndex(LONG* a_pnIndex) const = 0;


	virtual HRESULT STDMETHODCALLTYPE SingleScan(double a_dblFreq, ULONG a_nSamples, void* a_pData) = 0;
	
	virtual HRESULT STDMETHODCALLTYPE GetRaw(void** pRawSource) const = 0;

	virtual HRESULT STDMETHODCALLTYPE GetUtsChannel(void** pRawUts) const = 0;

	virtual HRESULT STDMETHODCALLTYPE BurnSettings(ULONG _Property) const = 0;


};

#endif