#ifndef IMEBIUSSCALE_H
#define IMEBIUSSCALE_H

#include "mebase.h"
#include "mestring.h"

namespace mebase{ namespace system{

typedef enum {
	MESCT_MULTIPLIER,
	MESCT_LINEAR,
	MESCT_INTERPOLATE,
	MESCT_POLYNOMIAL,
	MESCT_SUPER =0x0FFFF
	
}MESCALE_TYPES;

using mebase::MEBRESULT;

///     :
/// 1.   ,      . 
///    ,            
/// 2.       /,       
/// 3.   (=>)    ,
///   2  3  .
/// @interface IMebiusScale 
class IMebiusScale 
{
public:
	
	///   . <br>
	/// * : , 
	/// * 
	/// * r/o
	virtual MEBRESULT MEBCALLTYPE GetOptions(mebase::ULONG* pnOptions)const=0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE SetOptions(mebase::ULONG nOptions)=0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE GetName(mestring& msName)const=0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE SetName(const mestring& msName)=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE GetSrcUnits(mestring& msName)const=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE SetSrcUnits(const mestring& msName)=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE GetDstUnits(mestring& msName)const=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE SetDstUnits(const mestring& msName)=0;

	///   
	virtual MEBRESULT MEBCALLTYPE GetSrcRange(float* pMin, float* pMax)const=0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE SetSrcRange(float Min, float pMax)=0;

	///   
	virtual MEBRESULT MEBCALLTYPE GetDstRange(float* pMin, float* pMax)const=0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE SetDstRange(float Min, float pMax)=0;

	///   
	virtual MEBRESULT MEBCALLTYPE GetType(mebase::ULONG* pnType)const=0;
	
	///    
	virtual MEBRESULT MEBCALLTYPE GetTypeName(mestring& msName)const=0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE GetInfo(mestring& msInfo)const=0;
	
	///  
	///
	
	///  
	virtual MEBRESULT MEBCALLTYPE Eval(float Val, float* OutVal)const=0;
	
	///  
	virtual MEBRESULT MEBCALLTYPE EvalInverse(float Val, float* OutVal)const=0;

	/// ,     , -      .
	virtual MEBRESULT MEBCALLTYPE EvalEx(float Val, float* OutVal, mebase::ULONG* pnState)const=0;
	
	///   ,     , -      .
	virtual MEBRESULT MEBCALLTYPE EvalInverseEx(float Val, float* OutVal, mebase::ULONG* pnState)const=0;

	///   
	///
	
	virtual MEBRESULT MEBCALLTYPE Compare(const IMebiusScale* pScale, mebase::ULONG* result)=0;
	virtual MEBRESULT MEBCALLTYPE GetMathModule(void** ppObject)=0;	
	
};

class IMebiusScaleLinear
{
public:
	virtual MEBRESULT MEBCALLTYPE SetA(float a)=0;
	virtual MEBRESULT MEBCALLTYPE GetA(float* pA)const=0;

	virtual MEBRESULT MEBCALLTYPE SetB(float b)=0;
	virtual MEBRESULT MEBCALLTYPE GetB(float* pB)const=0;


};

class IMebiusScaleMultiplier
{
public:
	virtual MEBRESULT MEBCALLTYPE SetMultiplier(float mult)=0;
	virtual MEBRESULT MEBCALLTYPE GetMultiplier(float* pMult)const=0;
};

class IMebiusScaleInterpolate
{
public:
	virtual MEBRESULT MEBCALLTYPE GetNodesCount(mebase::ULONG * pnCount)const=0;
	virtual MEBRESULT MEBCALLTYPE GetNode(float* pfValX, float* pfValY, int nIndex)const=0;
	virtual MEBRESULT MEBCALLTYPE InsertNode(float fValX, float fValY, int* pnIndex)=0;
	virtual MEBRESULT MEBCALLTYPE DeleteNode(int nIndex)=0;


};

class IMebiusScalePolynomial
{
};

}}

#endif


